# Configuration File Guidelines 
This repository contains sample configuration files that can be used to run NAML. 
Here is how a configuration file is formatted.</br>

Each config file should have the following:
* `filePath` , a string, defines the path to the data you will be using
* `loggingEnabled`, true or false, determines whether the output of the run will be logged onto a text file 
* `targetCol` is a string that specifies which column the data will be classified into 
* `percentTrain` is a float value that determines the percent of data that will be trained on 
* `jobs` is a list of jobs that will be run. Within each job you must specify a method. If the type of method is univariate transformation, you can also specify a classifier.</br></br>
Here is an an example of a configuration file. This can also be found in [example_config.json](https://github.com/erins/NAML/blob/master/scripts/configFiles/example_config.json)

```
{
 "filePath":"../scripts/data/2013e.csv",
 "loggingEnabled": true,
 "targetCol": "event",
 "percentTrain":0.5,
 "jobs": [
    {"method": "SHAPELET_TRANSFORM"
    },
    {"method": "UNIVARIATE_TRANSFORMATION",
     "classifier": "TSF_CLF"
    },
    {"method": "COLUMN_ENSEMBLE",
     "ensembleInfo":[{
                        "classifier": "TSF_CLF",
                        "columnNum":1
                       },
                       {
                        "classifier": "TSF_CLF",
                        "columnNum":0
                       }]
    }
  ]
}
```

Within **jobs** there are three main methods that can be specified. 
* Concatenation Method used by specifying `"method": "UNIVARIATE_TRANSFORMATION"`
* Column Ensemble Method used by specifying `"method":"COLUMN_ENSEMBLE"`
* Multivariate Method used by specifying `"method":"SHAPELET_TRANSFORM"` </br> </br>

This program also provides *three univariate classifiers* that **must be specified** for the Concatenation Method and the Column Ensemble Method. The following shows the classifier and how to specify them within each job. 
* Time Series Forest Classifier `"classifier":"TSF_CLF"`
* K Neighbors Classifier `"classifier":"KNN_CLF"`
* Proximity Forest Classifier `"classifier":"PF_CLF"`</br> 

For the Column Ensemble Method, you must specify `ensembleInfo` in which you have to specify a `classifier` and `columnNum` per member of the ensemble. Each of these members of the ensemble will need to be specified within `{}`. </br> 

Lastly, there is also the option to parametrize each of these univariate classifiers and the multivariate method by specifying it in the following manner (within the job): </br></br>
`"parameters":[paramOneName, paramOneValue, paramTwoName, paramTwoValue....]`</br> 
An example of this can be seen in the following [JSON file](https://github.com/erins/NAML/tree/master/scripts/configFiles/parameters_example.json).</br>
```
{
 "filePath":"../scripts/data/sample-Sheet1.csv",
 "loggingEnabled": true,
 "targetCol": "event",
 "percentTrain":0.8,
 "jobs": [
    {"method": "UNIVARIATE_TRANSFORMATION",
     "classifier": "TSF_CLF",
     "parameters":["n_estimators",15,"min_samples_split",3, "min_samples_leaf",2]
    }
  ]
}
```

