#!/usr/bin/env python
# coding: utf-8

'''
configuration_checker.py
purpose:
1) To check if the configuration file is properly formatted
   * Is there anything that shouldn't be specified
   * Are there any values that shouldn't be in the data
2) Check if the data is formatted properly 
'''

import json
import sys
import pandas as pd 

required_keys=['filePath', 'loggingEnabled','targetCol', 'percentTrain', 'jobs']
u_classifiers=['TSF_CLF','KNN_CLF','PF_CLF']
headers=['name','event','channel','start time', 'end time']

#checking the jobs for proper methods and classifiers 
def check_job(j):
    job_keys=j.keys()
    if ('method' not in job_keys):
        print("you need to specify the method")
    else:
        method= j['method']
        classifier=""
        if (method == 'UNIVARIATE_TRANSFORMATION'):
            if('classifier' not in job_keys):
                print("A classifier must be specified for univariate transform method")
            else:
                classifier=j['classifier'] 
                if(classifier not in u_classifiers):
                    print("Classifier "+classifier+" is not implemented")
        elif (method=="SHAPELET_TRANSFORM"):
            if ("classifier" in  job_keys):
                print("classifier does not need to be specified for shapelet transform method")
        elif (method == 'COLUMN_ENSEMBLE'): 
            info=j['ensembleInfo']
            for e in info:
                if("columnNum" not in e.keys()):
                    print("A column number must be specified for the column ensemble method")
                if('classifier' not in e.keys()):
                    print("A classifier must be specified for univariate transform method")
                else:
                    classifier=e['classifier'] 
                    if(classifier not in u_classifiers):
                        print("Classifier "+classifier+" is not implemented")
        else:
            print("Method "+method +" is not supported")
                
            
def main():
    #read in json file 
    script = sys.argv[0]
    json_file_name = sys.argv[1]
    with open(json_file_name) as f:
        data = json.load(f)


    #check if the the keys are right 
    print("checking configuration file: "+json_file_name+"...\n")
    input_keys=data.keys()
    for e in input_keys:
        if e not in required_keys:
            print (e +" is not a valid key within your inputed json file.")
            print("Valid keys include:")
            print(str(input_keys)+"\n")


    #check the value of percentTrain within the json files 
    if "percentTrain" in input_keys:
        val= data['percentTrain']
        if (val <= 0 or val >= 1):
            print( "percentTrain must be between 0 or 1. You have: " + str(data['percentTrain']) +"\n" )
    else: 
        print("percentTrain must be specified.\n") 


    #check the value of targetCol in the incoming 
    if "targetCol" in input_keys:
        if (data['targetCol'] not in headers):
            print("targetCol: "+data['targetCol']+" is not supported. \n")
    else: 
        print("targetCol must be specified in JSON configuration. \n")
 

    #check the format/values of these jobs 
    if "jobs" in input_keys:
        all_jobs= data['jobs']
        for j in all_jobs:
            check_job(j)

    else: 
        print ("jobs must be specified. \n") 


    #checking the data file 
    print("checking data file...")
    if "filePath" in input_keys:
        file_name=data['filePath']
        X=pd.read_csv(file_name)
        for e in list(X)[0:5]:
            if e not in headers:
                print(e + " is not a supported column")
                print("supported columns include "+ str(headers))

    print("check complete")

main()
