# Introduction 
If you’re here you are probably already familiar with sktime. However, if you just happened to stumble upon this, you can learn more about the toolbox on its 
* [official page](https://www.turing.ac.uk/research/research-projects/sktime-toolbox-data-science-time-series)
* [github page](https://github.com/alan-turing-institute/sktime/)
* [documentation](https://alan-turing-institute.github.io/sktime/)<br/>

I’ve written this guide based on the installation process on a linux and mac machine.  Please let me know if you feel that this guide is incomplete or could use any more information. To see the steps required to run this on a window computer please follow the steps outline under *Windows* in the sktime [documentation](https://alan-turing-institute.github.io/sktime/installation.html) <br/>
## Downloading and creating an sktime environment 
The first component of this section outlines creating a virtual environment. *However*, if you already have python set up on your machine and are comfortable downloading sktime, please skip to the bolded step. NOTE: sktime is only compatible with a version of python higher than 3.6. <br/>
### Creating a virtual environment with anaconda
Although this isn’t necessary to create a virtual environment, it is highly recommended as it will allow you to manage your python libraries and projects more efficiently. To install anaconda visit the [website](https://www.anaconda.com/distribution/) and download the linux installer for ` python 3` . In your downloads directory, you will find a shell script named similar to the following: `Anaconda3-2019.10-Linux-x86_64.sh`<br/><br/>
Run this script by typing the following on the command line: <br/>
`bash ./Anaconda3-2019.10-Linux-x86_64.sh`<br/><br/>
You will be prompted to accept the license and agreement as the program starts. Next, type the following onto the command line prompt. <br/>
`source .bashrc` <br/><br/>
This command essentially refreshes the .bashrc file that was modified by the previous command that you ran. Now, we are ready to create a virtual environment. Run the following command on the commandline. Feel free to replace “yourenvname” with anything you would like.<br/>
`conda create -n yourenvname python=3.7 anaconda`<br/><br/>
Next activate your environment by typing the following: <br/>
`conda activate yourenvname` <br/><br/>
Now that you are in the environment you can install stime by running the following command:<br/> 
	***`pip install sktime`***<br/><br/>
That’s it! You have sktime on your computer. You can deactivate the environment by typing the following command. <br/>
	`conda deactivate` <br/><br/>
To ensure that you installed sktime within the virtual environment. You can type the following onto the command: <br/>
	`Ipython` <br/><br/>
Ipython is a command shell for interactive python programming. If you type “import sktime” and click enter, you will be able to check if the sktime module is properly installed. Type “quit()” to get out of the shell.<br/>
## Running your first sktime experiment 
Now that we know a little more about pandas and numpy, we are ready to start our own mini experiment. I would highly suggest looking at the tutorial available on the sktime documentation website.<br/>
### Finding a suitable dataset 
There is a great time series dataset archive at timeseriesclassification.com . This archive has both univariate and multivariate datasets that you can use. Additionally, these datasets are also formatted to work seamlessly with sktime.  For this mini-experiment I will use atrial fibrillation data that can be found [here](http://www.timeseriesclassification.com/description.php?Dataset=AtrialFibrillation) . Once you download and unzip the folder, you will find that there are file types .ts and .arff. These files are supported by sktime load functions.  Now that the data is loaded, we need to load the data set. 
### Loading the dataset 
I would suggest using jupyter notebook within this directory  as we step through this tutorial as it will allow you to explore your data as you get started. Activate your environment, and launch jupyter notebook by typing the following onto your terminal. <br/>
`>conda activate environmentname`<br/>
`>jupyter notebook`<br/>
Now create a new python3 notebook and name it!  <br/>
![img1](https://github.com/erins/NAML/blob/master/tutorial/imgs/jnb_image1.png)
![img2](https://github.com/erins/NAML/blob/master/tutorial/imgs/jnb_image2.png)
### Running Models 
More [tutorials](https://alan-turing-institute.github.io/sktime/examples.html) and API documentation can be found in the [documentation](https://alan-turing-institute.github.io/sktime/) page. In this tutorial, we will run through the concatenation method outlined in the cells below.</br>
![img3](https://github.com/erins/NAML/blob/master/tutorial/imgs/jnb_image3.png)
You may also choose to replace the TimeSeriesForest classifier with any other univariate classifier as such: 
![img4](https://github.com/erins/NAML/blob/master/tutorial/imgs/jnb_image4.png)
You may notice a Pipeline() function that runs the column￼ concatenation. This tool allows you to run other transformations on the data as well. If you are looking for more information on the functions provided the library viewing and searching the documentation and looking at the github page can help. Searching the function `KNeighborsTimeSeriesClassifier` will give you information on all the parameters that can be used with this function.  If you are trying to find the implementation you can use the import statement on top of the tutorial. For example, the k-neighbors implementation can be found in the folder `sktime> classifier > distance_based`




